/**
 * @author pastakhov, MusikAnimal, Bhsd and others
 * @license GPL-2.0-or-later
 * @see https://gerrit.wikimedia.org/g/mediawiki/extensions/CodeMirror
 */
import { Tag } from '@lezer/highlight';
import type { StreamParser, StringStream } from '@codemirror/language';
declare const tokens: {
    apostrophes: string;
    comment: string;
    convertBracket: string;
    convertDelimiter: string;
    convertFlag: string;
    convertLang: string;
    doubleUnderscore: string;
    em: string;
    error: string;
    extLink: string;
    extLinkBracket: string;
    extLinkProtocol: string;
    extLinkText: string;
    extTag: string;
    extTagAttribute: string;
    extTagAttributeValue: string;
    extTagBracket: string;
    extTagName: string;
    fileText: string;
    freeExtLink: string;
    freeExtLinkProtocol: string;
    hr: string;
    htmlEntity: string;
    htmlTagAttribute: string;
    htmlTagAttributeValue: string;
    htmlTagBracket: string;
    htmlTagName: string;
    imageParameter: string;
    linkBracket: string;
    linkDelimiter: string;
    linkPageName: string;
    linkText: string;
    linkToSection: string;
    list: string;
    magicLink: string;
    pageName: string;
    parserFunction: string;
    parserFunctionBracket: string;
    parserFunctionDelimiter: string;
    parserFunctionName: string;
    redirect: string;
    section: string;
    sectionHeader: string;
    signature: string;
    skipFormatting: string;
    strong: string;
    tableBracket: string;
    tableCaption: string;
    tableDefinition: string;
    tableDefinitionValue: string;
    tableDelimiter: string;
    tableDelimiter2: string;
    tableTd: string;
    tableTh: string;
    template: string;
    templateArgumentName: string;
    templateBracket: string;
    templateDelimiter: string;
    templateName: string;
    templateVariable: string;
    templateVariableBracket: string;
    templateVariableDelimiter: string;
    templateVariableName: string;
};
declare type Style = string | [string];
declare type Tokenizer<T = Style> = ((stream: StringStream, state: State) => T) & {
    args?: unknown[];
};
declare type NestCount = 'nTemplate' | 'nExt' | 'nVar' | 'nLink' | 'nExtLink';
declare interface Nesting extends Record<NestCount, number> {
    extName: string | false;
    extState: object | false;
}
declare interface State extends Nesting {
    tokenize: Tokenizer;
    readonly stack: Tokenizer[];
    readonly inHtmlTag: string[];
    extMode: StreamParser<object> | false;
    lbrack: boolean | undefined;
    bold: boolean;
    italic: boolean;
    dt: Partial<Nesting> & {
        n: number;
        html: number;
    };
    redirect: boolean;
    data: MediaWikiData;
}
declare interface Token {
    pos: number;
    readonly string: string;
    style: Style;
    readonly state: State;
}
export type TagName = keyof typeof tokens;
export type ApiSuggestions = [string, string?][];
/**
 * 获取维基链接建议
 * @param search 搜索字符串，开头不包含` `
 * @param namespace 命名空间
 * @param subpage 是否为子页面
 */
export type ApiSuggest = (search: string, namespace?: number, subpage?: boolean) => ApiSuggestions | Promise<ApiSuggestions>;
export interface MwConfig {
    readonly tags: Record<string, true>;
    readonly tagModes: Record<string, string>;
    urlProtocols: string;
    functionSynonyms: [Record<string, string>, Record<string, unknown>];
    doubleUnderscore: [Record<string, unknown>, Record<string, unknown>];
    nsid: Record<string, number>;
    variants?: string[];
    img?: Record<string, string>;
    redirection?: string[];
    permittedHtmlTags?: string[];
    implicitlyClosedHtmlTags?: string[];
    linkSuggest?: ApiSuggest;
    paramSuggest?: ApiSuggest;
}
declare class MediaWikiData {
    /** 已解析的节点 */
    readonly readyTokens: Token[];
    /** 当前起始位置 */
    oldToken: Token | null;
    /** 可能需要回滚的`'''` */
    mark: number | null;
    firstSingleLetterWord: number | null;
    firstMultiLetterWord: number | null;
    firstSpace: number | null;
    readonly tags: string[];
    constructor(tags: string[]);
}
/** Adapted from the original CodeMirror 5 stream parser by Pavel Astakhov */
export declare class MediaWiki {
    readonly config: MwConfig;
    readonly tokenTable: {
        [x: string]: Tag;
    };
    readonly hiddenTable: Record<string, Tag>;
    readonly permittedHtmlTags: Set<string>;
    readonly implicitlyClosedHtmlTags: Set<string>;
    readonly urlProtocols: RegExp;
    readonly linkRegex: RegExp;
    readonly fileRegex: RegExp;
    readonly redirectRegex: RegExp;
    readonly img: string[];
    readonly imgRegex: RegExp;
    readonly headerRegex: RegExp;
    readonly templateRegex: RegExp;
    readonly argumentRegex: RegExp;
    readonly convertSemicolon: RegExp;
    readonly convertLang: RegExp;
    readonly convertRegex: RegExp;
    readonly wikiRegex: RegExp;
    readonly tableDefinitionRegex: RegExp;
    readonly tags: string[];
    constructor(config: MwConfig);
    /**
     * Dynamically register a token in CodeMirror.
     * This is solely for use by this.addTag() and CodeMirrorModeMediaWiki.makeLocalStyle().
     *
     * @param token
     * @param hidden Whether the token is not highlighted
     * @param parent
     */
    addToken(token: string, hidden?: boolean, parent?: Tag): void;
    /**
     * Register the ground tokens. These aren't referenced directly in the StreamParser, nor do
     * they have a parent Tag, so we don't need them as constants like we do for other tokens.
     * See makeLocalStyle() for how these tokens are used.
     */
    registerGroundTokens(): void;
    inStr(str: string, tag: TagName | false, errorTag?: TagName): Tokenizer;
    eatWikiText(style: string): Tokenizer;
    eatApostrophes(obj: Pick<State, 'bold' | 'italic'>): Tokenizer<string | false>;
    eatExternalLinkProtocol(chars: string, free?: boolean): Tokenizer;
    inExternalLink(text?: boolean): Tokenizer;
    get eatFreeExternalLink(): Tokenizer;
    inLink(file: boolean, section?: boolean): Tokenizer;
    inLinkText(file: boolean, gallery?: boolean): Tokenizer;
    toEatImageParameter(stream: StringStream, state: State): void;
    eatList(stream: StringStream, state: State): string;
    eatDoubleUnderscore(style: string, stream: StringStream, state: State): Style;
    get eatStartTable(): Tokenizer;
    inTableDefinition(tr?: boolean, quote?: string): Tokenizer;
    get inTable(): Tokenizer;
    inTableCell(style: string, needAttr?: boolean, firstLine?: boolean): Tokenizer;
    inSectionHeader(str: string): Tokenizer;
    get inComment(): Tokenizer;
    eatTagName(name: string, isCloseTag?: boolean, isHtmlTag?: boolean): Tokenizer;
    inHtmlTagAttribute(name: string, quote?: string): Tokenizer;
    inExtTagAttribute(name: string, quote?: string, isLang?: boolean): Tokenizer;
    eatExtTagArea(name: string): Tokenizer;
    inExtTokens(origString: string): Tokenizer;
    inVariable(pos?: number): Tokenizer;
    eatTransclusion(stream: StringStream, state: State): string;
    inParserFunctionName(invoke?: boolean): Tokenizer;
    inTemplatePageName(haveEaten?: boolean, anchor?: boolean): Tokenizer;
    inParserFunctionArgument(module?: boolean): Tokenizer;
    inTemplateArgument(expectName?: boolean): Tokenizer;
    inConvert(style: string, needFlag?: boolean, needLang?: boolean): Tokenizer;
    eatEntity(stream: StringStream, style: string): string;
    /**
     * main entry
     *
     * @see https://codemirror.net/docs/ref/#language.StreamParser
     *
     * @param tags
     */
    mediawiki(tags?: string[]): StreamParser<State>;
    'text/nowiki'(): StreamParser<Record<string, never>>;
    inReferences(tag: string, comment?: boolean): Tokenizer<string>;
    'text/references'(tags: string[]): StreamParser<State>;
    'text/choose'(tags: string[]): StreamParser<State>;
    'text/combobox'(tags: string[]): StreamParser<State>;
    get inInputbox(): Tokenizer<string>;
    'text/inputbox'(tags: string[]): StreamParser<State>;
    inGallery(section?: boolean): Tokenizer;
    'text/gallery'(tags: string[]): StreamParser<State>;
}
export {};
